#!   /usr/bin/env	python
#    coding: utf8

import sys
import rr
import random
import time
import csr
import onewire
import ds18b20


class CSpecFmcAdc100Ms:

    SPEC_ONEWIRE_ADDR = 0x20000
    SPEC_CSR_ADDR = 0x30000
    UTC_CORE_ADDR = 0x40000
    IRQ_CONTROLLER_ADDR = 0x50000

    # SPEC CSR
    CSR_TYPE_VER = 0x00
    CSR_BSTM_TYPE = 0x04
    CSR_BSTM_DATE = 0x08
    CSR_STATUS = 0x0C
    CSR_CTRL = 0x10

    PCB_VER_MASK = 0x000F
    CARRIER_TYPE_MASK = 0xFFFF0000

    STATUS_FMC_PRES = (1<<0)
    STATUS_P2L_PLL_LCK = (1<<1)
    STATUS_SYS_PLL_LCK = (1<<2)
    STATUS_DDR3_CAL_DONE = (1<<3)

    CTRL_LED_GREEN = (1<<0)
    CTRL_LED_RED = (1<<1)
    CTRL_DAC_CLR_N = (1<<2)

    # UTC core
    UTC_CORE_SECONDS = 0x0
    UTC_CORE_COARSE = 0x4

    # tag = [wb_addr_offset, meta, second, coarse, fine]
    tag_trig = [0x8, 0, 0, 0, 0]
    tag_start = [0x18, 0, 0, 0, 0]
    tag_stop = [0x28, 0, 0, 0, 0]
    tag_end = [0x38, 0, 0, 0, 0]

    # IRQ controller
    IRQ_CTRL_MULT = 0x0
    IRQ_CTRL_SRC = 0x4
    IRQ_CTRL_EN_MASK = 0x8


    def __init__(self, bus):
        self.bus = bus
        self.spec_onewire = onewire.COpenCoresOneWire(self.bus, self.SPEC_ONEWIRE_ADDR, 624, 124)
        self.ds18b20 = ds18b20.CDS18B20(self.spec_onewire, 0)
        self.spec_csr = csr.CCSR(self.bus, self.SPEC_CSR_ADDR)
        self.utc_core = csr.CCSR(self.bus, self.UTC_CORE_ADDR)
        self.irq_controller = csr.CCSR(self.bus, self.IRQ_CONTROLLER_ADDR)

    def __del__(self):
        pass

    # print SPEC unique ID
    def print_unique_id(self):
        print('SPEC unique ID: %.12X') % self.ds18b20.read_serial_number()

    # print SPEC temperature
    def print_temp(self):
        serial_number = self.ds18b20.read_serial_number()
        print("SPEC temperature: %3.3f°C") % self.ds18b20.read_temp(serial_number)

    # Returns SPEC unique ID
    def get_unique_id(self):
        return self.ds18b20.read_serial_number()

    # Returns SPEC temperature
    def get_temp(self):
        serial_number = self.ds18b20.read_serial_number()
        if(serial_number == -1):
            return -1
        else:
            return self.ds18b20.read_temp(serial_number)

    # Set UTC core with current computer time
    def set_utc_time(self):
        current_time = time.time()
        utc_seconds = int(current_time)
        self.utc_core.wr_reg(self.UTC_CORE_SECONDS, utc_seconds)
        utc_coarse = int((current_time - utc_seconds)/8E-9)
        self.utc_core.wr_reg(self.UTC_CORE_COARSE, utc_coarse)
        return curent_time

    # Returns UTC seconds counter value
    def get_utc_second_cnt(self):
        return self.utc_core.rd_reg(self.UTC_CORE_SECONDS)

    # Set UTC seconds counter
    def set_utc_second_cnt(self, value):
        self.utc_core.wr_reg(self.UTC_CORE_SECONDS, value)
        return self.utc_core.rd_reg(self.UTC_CORE_SECONDS)

    # Returns UTC coarse counter value
    def get_utc_coarse_cnt(self):
        return self.utc_core.rd_reg(self.UTC_CORE_COARSE)

    # Set UTC coarse counter
    def set_utc_coarse_cnt(self, value):
        self.utc_core.wr_reg(self.UTC_CORE_COARSE, value)
        return self.utc_core.rd_reg(self.UTC_CORE_COARSE)

    # Returns last trigger event time-tag
    def get_utc_trig_tag(self):
        # Get metadata
        addr = self.tag_trig[0]
        self.tag_trig[1] = self.utc_core.rd_reg(addr)
        # Get seconds
        addr = self.tag_trig[0]+0x4
        self.tag_trig[2] = self.utc_core.rd_reg(addr)
        # Get coarse
        addr = self.tag_trig[0]+0x8
        self.tag_trig[3] = self.utc_core.rd_reg(addr)
        # Get fine
        addr = self.tag_trig[0]+0xc
        self.tag_trig[4] = self.utc_core.rd_reg(addr)

        return self.tag_trig

    # Returns last acquisition start event time-tag
    def get_utc_start_tag(self):
        # Get metadata
        addr = self.tag_start[0]
        self.tag_start[1] = self.utc_core.rd_reg(addr)
        # Get seconds
        addr = self.tag_start[0]+0x4
        self.tag_start[2] = self.utc_core.rd_reg(addr)
        # Get coarse
        addr = self.tag_start[0]+0x8
        self.tag_start[3] = self.utc_core.rd_reg(addr)
        # Get fine
        addr = self.tag_start[0]+0xc
        self.tag_start[4] = self.utc_core.rd_reg(addr)

        return self.tag_start

    # Returns last acquisition stop event time-tag
    def get_utc_stop_tag(self):
        # Get metadata
        addr = self.tag_stop[0]
        self.tag_stop[1] = self.utc_core.rd_reg(addr)
        # Get seconds
        addr = self.tag_stop[0]+0x4
        self.tag_stop[2] = self.utc_core.rd_reg(addr)
        # Get coarse
        addr = self.tag_stop[0]+0x8
        self.tag_stop[3] = self.utc_core.rd_reg(addr)
        # Get fine
        addr = self.tag_stop[0]+0xc
        self.tag_stop[4] = self.utc_core.rd_reg(addr)

        return self.tag_stop

    # Returns last acquisition end event time-tag
    def get_utc_end_tag(self):
        # Get metadata
        addr = self.tag_end[0]
        self.tag_end[1] = self.utc_core.rd_reg(addr)
        # Get seconds
        addr = self.tag_end[0]+0x4
        self.tag_end[2] = self.utc_core.rd_reg(addr)
        # Get coarse
        addr = self.tag_end[0]+0x8
        self.tag_end[3] = self.utc_core.rd_reg(addr)
        # Get fine
        addr = self.tag_end[0]+0xc
        self.tag_end[4] = self.utc_core.rd_reg(addr)

        return self.tag_end

    # Set IRQ enable mask
    def set_irq_en_mask(self, mask):
        self.irq_controller.wr_reg(self.IRQ_CTRL_EN_MASK, mask)
        return self.irq_controller.rd_reg(self.IRQ_CTRL_EN_MASK)

    # Get IRQ enable mask
    def get_irq_en_mask(self):
        return self.irq_controller.rd_reg(self.IRQ_CTRL_EN_MASK)

    # Returns multiple IRQ status
    def get_irq_mult(self):
        return self.irq_controller.rd_reg(self.IRQ_CTRL_MULT)

    # Clears multiple IRQ status
    def clear_irq_mult(self, irq):
        self.irq_controller.wr_reg(self.IRQ_CTRL_MULT, irq)
        return self.irq_controller.rd_reg(self.IRQ_CTRL_MULT)

    # Returns IRQ source
    def get_irq_source(self):
        return self.irq_controller.rd_reg(self.IRQ_CTRL_SRC)

    # Clears IRQ source
    def clear_irq_source(self, irq):
        self.irq_controller.wr_reg(self.IRQ_CTRL_SRC, irq)
        return self.irq_controller.rd_reg(self.IRQ_CTRL_SRC)


